/* Generated by wayland-scanner 1.22.0 */

#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_buffer_interface;
extern const struct wl_interface wl_output_interface;

static const struct wl_interface *weston_screenshooter_types[] = {
	&wl_output_interface,
	&wl_buffer_interface,
};

static const struct wl_message weston_screenshooter_requests[] = {
	{ "shoot", "oo", weston_screenshooter_types + 0 },
};

static const struct wl_message weston_screenshooter_events[] = {
	{ "done", "", weston_screenshooter_types + 0 },
};

WL_PRIVATE const struct wl_interface weston_screenshooter_interface = {
	"weston_screenshooter", 1,
	1, weston_screenshooter_requests,
	1, weston_screenshooter_events,
};

